// Author: George W. Taylor
// http://www.tropicalcoder.com/
// Date: 15 Oct 2012
// License: Creative Commons - Non-Commercial Use

// Debug routines
#include <Windows.h>
#include <time.h>
#include <sys\types.h>
#include <sys\timeb.h>
#include <stdlib.h>
#include "Debug.h"


//---------------------------------------------------------------------------

//void InitDebugFile(char *sDebugFilePathname)
void DebugLogging::InitDebugFile(void)
{
    remove("MFTP2Dbg.Log");
    rename("MFTPDbg1.Log", "MFTPDbg2.Log");
    rename("MFTPDbg.Log", "MFTPDbg1.Log");
    fpDebug = fopen("MFTPDbg.Log", "wt");
    if(fpDebug != NULL)
	{
	   hMutex = CreateMutex(NULL,FALSE,NULL);
       fputs("Debug log\r\n\r\n", fpDebug);
       fputs("Date: ", fpDebug);
	   OutputDateString();
       fputs("- Time started: ", fpDebug);
       OutPutTimeString();
       fputs("\r\n", fpDebug);
    }
}

void DebugLogging::CloseDebugFile(void)
{
 if(fpDebug != NULL)
 {
    fputs("\n\nShutting down debug logging system...\n", fpDebug);
    fputs("Date: ", fpDebug);
	OutputDateString();
    fputs("- Time: ", fpDebug);
    OutPutTimeString();
    fputs("Goodbye\n", fpDebug);
	fflush(fpDebug);
	fclose(fpDebug);
	fpDebug = NULL;
    if(hMutex != NULL)
	{ 	
       WaitForSingleObject(hMutex, INFINITE);
       CloseHandle(hMutex); 
    }
 }
}


void DebugLogging::DebugMessage(const char* msg)
{
 char sMessage[1024];

 if(hMutex != NULL)WaitForSingleObject(hMutex, INFINITE);

 if(fpDebug != NULL)
 {
    strcpy(sMessage, msg);
    strcat(sMessage, "\n");
    fputs(sMessage, fpDebug);
    fflush(fpDebug);
 }

 if(hMutex != NULL)ReleaseMutex(hMutex);
}

void DebugLogging::OutputDateString(void)
{
 char DateString[64];
 struct tm *today;
 time_t ltime;

 if(hMutex != NULL)WaitForSingleObject(hMutex, INFINITE);

 time(&ltime); // Gets time in seconds since UTC 1/1/70
 today = localtime( &ltime ); // Convert to time structure

 DateString[0] = '\0';
 // Use strftime to build a customized date string...
 strftime(DateString, 64, "%A, %B/%d/%Y ", today);
 // Date: Friday, January/21/2000 - Time started: 20:54:03 470
 // 01222000 = January 22 2000
 fputs(DateString, fpDebug);

 if(hMutex != NULL)ReleaseMutex(hMutex);
}


// Assumes a valid file pointer!
void DebugLogging::OutPutTimeString(void)
{
 char TimeStr[128];
 char numstr[8];
 struct tm *today;
 struct _timeb tstruct;
 time_t ltime;

 if(hMutex != NULL)WaitForSingleObject(hMutex, INFINITE);

 time(&ltime); // Gets time in seconds since UTC 1/1/70
 today = localtime( &ltime ); // Convert to time structure
 _ftime( &tstruct );
 
 // Build the string...
 *TimeStr = '\0';


 // hours number 0 - 23 (hours since midnight)
 if(today->tm_hour == 0)
 {
    strcat(TimeStr, "00");
 }else
 {
    if(today->tm_hour < 10)
    {
       TimeStr[0] = '0';
       TimeStr[1] = '\0';
    }
    _itoa(today->tm_hour, numstr, 10);
	strcat(TimeStr, numstr);
 }
 TimeStr[2] = ':';
 TimeStr[3] = '\0';

 // minutes number 0 - 59
 if(today->tm_min == 0)
 {
    strcat(TimeStr, "00");
 }else
 {
    if(today->tm_min < 10)
    {
       TimeStr[3] = '0';
       TimeStr[4] = '\0';
    }
    _itoa(today->tm_min, numstr, 10);
	strcat(TimeStr, numstr);
 }
 TimeStr[5] = ':';
 TimeStr[6] = '\0';

 
 // seconds number 0 - 59
 if(today->tm_sec == 0)
 {
    strcat(TimeStr, "00");
 }else
 {
    if(today->tm_sec < 10)
    {
       TimeStr[6] = '0';
       TimeStr[7] = '\0';
    }
    _itoa(today->tm_sec, numstr, 10);
	strcat(TimeStr, numstr);
 }

 TimeStr[8] = ' ';
 TimeStr[9] = '\0';

 // leaving a space before "milliseconds"...

   // Get the milliseconds 0 - 999
   _itoa(tstruct.millitm, numstr, 10);
   if(tstruct.millitm < 10)
   {
      TimeStr[9] = '0';
      TimeStr[10] = '0';
      TimeStr[11] = '\0';
   }else if(tstruct.millitm < 100)
   {
      TimeStr[9] = '0';
      TimeStr[10] = '\0';
   }else TimeStr[11] = '\0';

   strcat(TimeStr, numstr);
   strcat(TimeStr, "\n");
 
 fputs(TimeStr, fpDebug);

 if(hMutex != NULL)ReleaseMutex(hMutex);
}

