// Author: George W. Taylor
// http://www.tropicalcoder.com/
// Date: 25 May 2024
// License: This work is licensed under Creative Commons Attribution-ShareAlike 4.0 International 
// (CC BY-SA 4.0). https://creativecommons.org/licenses/by-sa/4.0/

// Receiver.cpp : Implements the Minimalist File Transfer Protocol (MFTP) receiver application.

#include <winsock2.h>
#include <ws2tcpip.h>
#include <Windows.h>
#include "Server.h"
#include "CheckCRC.h"
#include "Debug.h"
#include "UDP_Send.h"
#include "UDP_Receive.h"
#include <vector>
#include <string>
#include <iostream>
#include <fstream>

// Link with Ws2_32.lib
#pragma comment(lib, "Ws2_32.lib")

CheckCRC* pCRC;
DebugLogging* pDebug;
Server* pServer;
UDP_Send* pUDPSend;
UPD_Receive* pUDPReceive;
PCSTR address = "127.0.0.1";
u_short SendPort = 9789;
u_short RcvrPort = 9788;

std::string filename;
std::vector<uint8_t> file_data;



/*
// CRC function type
using CRCFunction = std::function<uint32_t(const std::vector<uint8_t>&)>;

// Logging function type
using LoggingFunction = std::function<void(const std::string&)>;

// UDP send function type
using UDPSendFunction = std::function<void(const std::vector<uint8_t>&)>;

// UDP receive function type
using UDPReceiveFunction = std::function<std::vector<uint8_t>()>;

*/

// Create a wrapper for the CRC32 calculation
static uint32_t exampleCRCFunction(char* pBuf, uint32_t countBytes)
{
	if (pCRC)
	{
		return pCRC->getCRC32(pBuf, countBytes);
	}
	return 0;
}


// Create a wrapper for the logging function
static void exampleLoggingFunction(const std::string& message)
{
	if (pDebug)
	{
		pDebug->DebugMessage((char*)message.c_str());
	}
}

static void exampleUDPSendFunction(const std::vector<uint8_t>& data)
{
	if (pUDPSend)
	{
		pUDPSend->exampleUDPSendFunction(data);
	}
}

static std::vector<uint8_t> exampleUDPReceiveFunction(bool* pbSuccess)
{
	if (pUDPReceive)
	{
		return pUDPReceive->exampleUDPReceiveFunction(pbSuccess);
	}
	return std::vector<uint8_t>();
}

int main()
{
	// Initialize Winsock
	WSADATA wsaData;
	int result = WSAStartup(MAKEWORD(2, 2), &wsaData);
	if (result != 0)
	{
		std::cerr << "WSAStartup failed with error: " << result << std::endl;
		return 1;
	}

	pCRC = new CheckCRC();

	pDebug = new DebugLogging();
	pDebug->InitDebugFile();

	pUDPSend = new UDP_Send(address, SendPort);

	pUDPReceive = new UPD_Receive(address, RcvrPort);

	pServer = new Server(exampleCRCFunction, exampleLoggingFunction, exampleUDPSendFunction, exampleUDPReceiveFunction);

	// Receive the file
	if (pServer->receiveFile(filename, file_data))
	{
 		std::cout << "File received successfully!" << std::endl;

		// Now filename and file_data should be populated
		std::cout << "Filename: " << filename << std::endl;
		std::cout << "File data size: " << file_data.size() << " bytes" << std::endl;

		// Write the file data to the specified filename
		std::ofstream output_file(filename, std::ios::binary);
		if (output_file.is_open())
		{
			output_file.write(reinterpret_cast<const char*>(file_data.data()), file_data.size());
			output_file.close();
			std::cout << "File written successfully." << std::endl;
		}
		else
		{
			std::cerr << "Failed to open file for writing." << std::endl;
		}
	}
	else
	{
		std::cout << "Failed to receive file!" << std::endl;
	}



	pDebug->CloseDebugFile();
	delete pCRC;
	delete pDebug;
	delete pUDPSend;
	delete pUDPReceive;
	delete pServer;
	return 0;
}
