// CheckCRC.cpp

#include "CheckCRC.h"

// CRC-32 used by V.42, Ethernet, SATA, MPEG-2, Gzip, PKZIP, POSIX cksum, PNG[24], ZMODEM
#define P_32 0xEDB88320L // using reversed


CheckCRC::CheckCRC()
{
 ZeroMemory(crc_tab32, TABLE_SIZE);
 init_crc32_tab();
}

CheckCRC::~CheckCRC()
{
}


DWORD CheckCRC::getCRC32(char *pBuf, uint32_t countBytes)
{
 DWORD crc_32 = 0xffffffff;

 for(uint32_t i = 0; i < countBytes; i++)
 {
     crc_32 = update_crc_32(crc_32, *pBuf++);
 }
 return crc_32 ^ 0xffffffff;
}


// From...
//    *   Library         : lib_crc
//    *   File            : tst_crc.c
//    *   Author          : Lammert Bies  1999-2008
//    *   E-mail          : info@lammertbies.nl
//    *   http://www.lammertbies.nl/

void CheckCRC::init_crc32_tab(void)
{
 DWORD crc;
 int i, j;

 for(i = 0; i < TABLE_SIZE; i++)
 {
     crc = (DWORD)i;
     for(j = 0; j < 8; j++)
	 {
         if(crc & 0x00000001L)
		 { 
            crc = (crc >> 1) ^ P_32;
		 }else crc >>= 1;
      }
      crc_tab32[i] = crc;
 }
}


// Ensure table was initialized first!
DWORD CheckCRC::update_crc_32(DWORD crc, char c)
{
 DWORD tmp, long_c;

 long_c = 0x000000ffL & (DWORD)c;
 tmp = crc ^ long_c;
 crc = (crc >> 8) ^ crc_tab32[ tmp & 0xff ];

 return crc;
}


   
