// Author: George W. Taylor
// http://www.tropicalcoder.com/
// Date: 25 May 2024
// License: This work is licensed under Creative Commons Attribution-ShareAlike 4.0 International 
// (CC BY-SA 4.0). https://creativecommons.org/licenses/by-sa/4.0/

#include "MFTP.h"

constexpr uint32_t MAX_TIMEOUT = 32000;         // WaitForSingleObject timeout in milliseconds

enum ACK_RESULT
{
	ACK_OK = 0,
	ACK_INVALID = 1,
	ACK_FOR_PREVIOUS_BLOCK = 2,
};

class Client : public MFTP
{
public:
	Client(CRCFunction crcFunc, LoggingFunction logFunc, UDPSendFunction udpSendFunc, UDPReceiveFunction udpReceiveFunc)
	: MFTP(crcFunc, logFunc, udpSendFunc, udpReceiveFunc) {}


 bool sendFile(const std::string& filename, LPBYTE& file_data, uint32_t &file_size);
 friend DWORD WINAPI RcvrThread(void* param);

private:
 ACK_RESULT VerifyMessage(const std::vector<uint8_t>& data, uint32_t blocknum, uint32_t crc32, uint32_t previous_crc32);
 bool isSameData(LPBYTE pSendBuf, LPBYTE pPrevBuf);
 bool AwaitAcknowlegment(const std::vector<uint8_t>& serializedMessage);
 bool HandleResends(const std::vector<uint8_t>& serializedMessage);
 void CleanupThread(void);
 std::vector<uint8_t> serializedMessage;
 HANDLE hThread = 0;
 HANDLE hEvent = 0;
 uint32_t count_blocks_sent = 0;
 uint32_t totalWaitMS = 0;
 bool bThreadLive = false;
 bool bAckRecd = false;
 bool bAckValid = false;
 bool bResend = false;
};

