// Author: George W. Taylor
// http://www.tropicalcoder.com/
// Date: 25 May 2024
// License: This work is licensed under Creative Commons Attribution-ShareAlike 4.0 International 
// (CC BY-SA 4.0). https://creativecommons.org/licenses/by-sa/4.0/


#include <Windows.h>
#include "FileLoader.h"

const DWORD FILE_OPEN_ERROR = 0xFFFFFFFF;

FileLoader::FileLoader(char *sPathName, bool bReadOnly)
{
 SECURITY_ATTRIBUTES secatt;
 secatt.nLength = sizeof(secatt);
 secatt.lpSecurityDescriptor = (LPVOID)NULL;
 secatt.bInheritHandle = true;

 hFile = 0;
 hMap = 0;
 dwFileSize = 0;
 pFile = NULL;

 // Use memory-mapped file IO to get a pointer to the disk file.
 // If MapViewOfFile() succeeds, pFile is a valid pointer.
 if(bReadOnly)
 {
    hFile = CreateFileA((LPCSTR)sPathName, // (LPCTSTR)
                    GENERIC_READ, FILE_SHARE_READ, &secatt,
                    OPEN_EXISTING, FILE_ATTRIBUTE_READONLY |
					FILE_FLAG_SEQUENTIAL_SCAN, NULL);
 }else
 {
    hFile = CreateFileA((LPCSTR)sPathName,
                    GENERIC_READ | GENERIC_WRITE, 0, &secatt,
                    CREATE_ALWAYS, FILE_ATTRIBUTE_NORMAL | FILE_FLAG_SEQUENTIAL_SCAN, NULL);
 }

 if((hFile == NULL) || (hFile == INVALID_HANDLE_VALUE))
 {
	 return;
 }

 if(bReadOnly)
 {
  dwFileSize = GetFileSize(hFile, NULL); // kernel32.lib
  if(dwFileSize == FILE_OPEN_ERROR)
  { 
     CloseHandle(hFile);
 	 hFile = 0;
	 dwFileSize = 0;
     return;
  }
 }


 if(bReadOnly)
 {
    hMap = CreateFileMappingA(hFile, &secatt, PAGE_READONLY, 0, 0, NULL);
    if(hMap == NULL)
	{
	   CloseHandle(hFile);
	   hFile = 0;
	   dwFileSize = 0;
       return;
	}
    pFile = (LPBYTE)MapViewOfFile(hMap, FILE_MAP_READ, 0, 0, 0);
 }else
 {
    hMap = CreateFileMappingA(hFile, &secatt, SEC_COMMIT | PAGE_READWRITE, 0, 0x100000, NULL);
    if(hMap == NULL)
	{
       CloseHandle(hFile);
	   hFile = 0;
	   dwFileSize = 0;
       return;
	}
    pFile = (LPBYTE)MapViewOfFile(hMap, FILE_MAP_WRITE, 0, 0, 0);
 }

 if(pFile == NULL)
 {
    CloseHandle(hMap);
    CloseHandle(hFile);
    hFile = hMap = 0;
    dwFileSize = 0;
    return;
 }

 strcpy(PathName, sPathName);
}

FileLoader::~FileLoader()
{
 if(!dwFileSize)return;
 if(hFile != NULL)
 {
	if(pFile != NULL)UnmapViewOfFile(pFile);
    if(hMap != NULL)CloseHandle(hMap);
	CloseHandle(hFile);
 }
 dwFileSize = 0;
}

char* FileLoader::GetPathName(void)
{
 return PathName;
};


